/**
 Tiny, basic from-scratch OpenGL and GLSL program.
 
 Dr. Orion Lawlor, olawlor@acm.org, 2008-01-28 (Public Domain)
*/
#include <GL/glew.h> /*<- for gl...ARB extentions.  Must call glewInit! */
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h> /* for "exit" */
#include <iostream>
#include <fstream>

/******************** Silly GLSL Boilerplate *********************/
// Print an error and exit if this object had a compile error.
void checkShaderOp(GLhandleARB obj,int errtype,const char *where) 
{
        GLint compiled;
        glGetObjectParameterivARB(obj,errtype,&compiled);
        if (!compiled) {
                printf("Compile error on program: %s\n",where);
                enum {logSize=10000};
                char log[logSize]; int len=0;
                glGetInfoLogARB(obj, logSize,&len,log);
                printf("Error Log: \n%s\n",log); exit(1);
        }
}
// Create a vertex or fragment shader from this code.
GLhandleARB makeShaderObject(int target,const char *code)
{
	GLhandleARB h=glCreateShaderObjectARB(target);
	glShaderSourceARB(h,1,&code,NULL);
	glCompileShaderARB(h);
	checkShaderOp(h,GL_OBJECT_COMPILE_STATUS_ARB,code);
	return h;
}
// Create a complete shader object from these chunks of GLSL shader code.
//  You still need to glUseProgramObjectARB(return value);
//  THIS IS THE FUNCTION YOU PROBABLY *DO* WANT TO CALL!!!!  RIGHT HERE!!!!
GLhandleARB makeProgramObject(const char *vertex,const char *fragment)
{
	if (glUseProgramObjectARB==0) 
	{ /* glew never set up, or OpenGL is too old.. */
		std::cout<<"Error!  OpenGL hardware or software too old--no GLSL!\n";
		exit(1);
	}
	GLhandleARB p=glCreateProgramObjectARB();
	glAttachObjectARB(p,
		makeShaderObject(GL_VERTEX_SHADER_ARB,vertex));
	glAttachObjectARB(p,
		makeShaderObject(GL_FRAGMENT_SHADER_ARB,fragment));
	glLinkProgramARB(p);
	checkShaderOp(p,GL_OBJECT_LINK_STATUS_ARB,"link");
	return p;
}
// Read an entire file into a C++ string.
std::string readFileIntoString(const char *fName) {
	char c; std::string ret;
	std::ifstream f(fName);
	if (!f) {ret="Cannot open file ";ret+=fName; return ret;}
	while (f.read(&c,1)) ret+=c;
        return ret;
}
// Create a complete shader object from these GLSL files.
GLhandleARB makeProgramObjectFromFiles(const char *vFile="vertex.txt",
	const char *fFile="fragment.txt")
{
	return makeProgramObject(
		readFileIntoString(vFile).c_str(),
		readFileIntoString(fFile).c_str()
	);
}
/********************* Real problem code *****************/
int mouse_x, mouse_y;

float randFloat(void) {
	return (rand()&0xffff)*(1.0/0xffff);
}

void my_display(void) {
	glClearColor(0,0,0,0); /* erase screen to black */
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	
	glLoadIdentity();
	glRotatef(mouse_x,0,1,0);
	glRotatef(mouse_y,1,0,0);
	
	static GLhandleARB prog=makeProgramObject(
	"//GLSL Vertex shader\n"
	"varying vec4 myColor; /*<- goes to fragment shader */ \n"
	"void main(void) {\n"
	"	myColor = gl_Color; \n"
	"	gl_Position=gl_ModelViewProjectionMatrix * gl_Vertex;\n"
	"}\n"
	,
	"//GLSL Fragment shader\n"
	"varying vec4 myColor; /*<- comes from vertex shader */ \n"
	"void main(void) {\n"
	"	gl_FragColor=myColor;\n"
	"}\n"
	);
	glUseProgramObjectARB(prog);
	
	/* Draw a random knot of line segments */
	srand(0);
	glBegin(GL_LINE_LOOP); // TRIANGLE_STRIP);
	for (int i=0;i<1000;i++) {
		glColor3f(randFloat(),randFloat(),randFloat());
		glVertex3f(randFloat()*2.0f-1.0f,
		           randFloat()*2.0f-1.0f,
			   randFloat()*2.0f-1.0f); 
	}
	glEnd();
	glutSwapBuffers();
	glutPostRedisplay();
}

void my_motion(int x,int y) 
{ 
	mouse_x=x; mouse_y=y; 
	glutPostRedisplay();
}

int main(int argc,char *argv[]) {
	glutInitWindowSize(600,600);
	glutInit(&argc,argv);
	int mode=GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
        glutInitDisplayMode (mode);
	glutCreateWindow("Demoland");
	glewInit(); /* <- muy importante!  Otherwise gl...ARB funcs just segfault! */
	
	glutDisplayFunc(my_display);
	glutMotionFunc(my_motion);
	glutMainLoop();
	return 0;
}
